@@ -454,7 +454,10 @@ def zOrder(x, y, minX, minY, size):
     # coords are transformed into non-negative 15-bit integer range
     x = 32767 * (x - minX) // size
     y = 32767 * (y - minY) // size
-
+    
+    x = int(x)
+    y = int(y)
+    
     x = (x | (x << 8)) & 0x00FF00FF
     x = (x | (x << 4)) & 0x0F0F0F0F
     x = (x | (x << 2)) & 0x33333333
