@@ -24,27 +24,32 @@ import numpy as np
 
 from . import get
 from . import create
-from . import utility
-from . import geom
 from . import earcut
 from . import calculate
+from . import geom
 from . import topobj
+from . import utility
 
-def fuse_vertices(vertex_list):
+
+def fuse_vertices(vertex_list: list[topobj.Vertex], decimals: int = 6) -> list[topobj.Vertex]:
     """
     This function fuses the vertices, duplicate vertices are fuse into a single vertex.
     
     Parameters
     ----------
-    vertex_list : a list of vertex
+    vertex_list : list[topobj.Vertex]
         A list of vertex topology to be fused.
- 
+    
+    decimals : int, optional
+        the precision of the calculation. Default = 6
+        
     Returns
     -------
-    fused_vertex_list : ndarray of vertex topology
+    fused_vertex_list : list[topobj.Vertex]
         A numpy array of the fused vertices
     """
     xyz_list = np.array([v.point.xyz for v in vertex_list])
+    xyz_list = np.round(xyz_list, decimals = decimals)
     vals, u_ids, inverse = np.unique(xyz_list, axis = 0,
                                       return_inverse=True,
                                       return_index=True)
@@ -58,25 +63,29 @@ def fuse_vertices(vertex_list):
         vertex_list[d_id[0]].update_attributes(new_dict)
         
     u_ids = np.sort(u_ids)
-    unique_v = vertex_list[u_ids]
+    unique_v = np.take(vertex_list, u_ids, axis=0)
     return unique_v
 
-def fuse_points(point_list):
+def fuse_points(point_list: list[geom.Point], decimals: int = 6) -> list[geom.Point]:
     """
     This function fuses the points, duplicate points are fuse into a single point.
     
     Parameters
     ----------
-    point_list : a list of point
+    point_list : list[geom.Point]
         A list of point object to be fused.
- 
+    
+    decimals : int, optional
+        the precision of the calculation. Default = 6
+
     Returns
     -------
-    fused_point_list : ndarray of point object
+    fused_point_list : list[geom.Point]
         A numpy array of the fused points
     """
     
     xyz_list = np.array([point.xyz for point in point_list])
+    xyz_list = np.round(xyz_list, decimals = decimals)
     vals, u_ids, inverse = np.unique(xyz_list, axis = 0,
                                       return_inverse=True,
                                       return_index=True)        
@@ -84,13 +93,13 @@ def fuse_points(point_list):
     unique_p = point_list[u_ids]
     return unique_p
 
-def triangulate_face(face, indices = False):
+def triangulate_face(face: topobj.Face, indices: bool = False) -> list[topobj.Face]:
     """
     Triangulates a face.
  
     Parameters
     ----------
-    face : topo object
+    face : topobj.Face
         the face object to be triangulated.
         
     indices : bool, optional
@@ -98,7 +107,7 @@ def triangulate_face(face, indices = False):
  
     Returns
     -------
-    list of face : list of face
+    list of face : list[topobj.Face]
         A list of triangles constructed from the meshing.
     """
     srf_type = face.surface_type
@@ -226,19 +235,19 @@ def triangulate_face(face, indices = False):
                 indx.append(tri2)
             return [np.array(gpts), np.array(indx)]
             
-def faces2mesh(face_list):
+def faces2mesh(face_list: list[topobj.Face]) -> dict:
     """
     This function converts faces to a mesh for visualisation.
  
     Parameters
     ----------
-    face_list : list of face object
+    face_list : list[topobj.Face]
         the list of face object to be triangulated.The face must be single facing, it
         cannot have regions that are facing itself.
  
     Returns
     -------
-    mesh_dictionary : dictionary
+    mesh_dictionary : dict
         vertices: ndarray of shape(Npoints,3) of the vertiices
         indices: ndarray of shape(Ntriangles,3) indices of the triangles.
     """
@@ -248,7 +257,11 @@ def faces2mesh(face_list):
     for f in face_list:
         srf_type = f.surface_type
         if srf_type == geom.SrfType.POLYGON or srf_type == geom.SrfType.BSPLINE:
+            # vs = get.vertices_frm_face(f)
+            # xyzs = [v.point.xyz for v in vs]
+            # print(xyzs)
             xyzs_indxs = triangulate_face(f, indices = True)
+            # print(xyzs_indxs)
             if len(all_xyzs) == 0:
                 all_xyzs = xyzs_indxs[0]
             else:
@@ -259,24 +272,56 @@ def faces2mesh(face_list):
             else:
                 indxs_new =  xyzs_indxs[1] + idx_cnt
                 all_idxs = np.append(all_idxs, indxs_new, axis=0)
-            
+        
             idx_cnt += len(xyzs_indxs[0])
         
     return {"vertices":all_xyzs, "indices":all_idxs}
 
-def edges2lines(edge_list):
+def edges2lineedges(edge_list: list[topobj.Edge]) -> list[topobj.Edge]:
+    """
+    converts edges to line edges.
+ 
+    Parameters
+    ----------
+    edge_list : list[topobj.Edge]
+        the list of edge object to convert to line edges.
+ 
+    Returns
+    -------
+    line_edges : list[topobj.Edge]
+        the list of converted line edges.
+    """
+    line_edge_list = []
+    
+    for e in edge_list:
+        vs = get.vertices_frm_edge(e)
+        if len(vs) == 2:
+            line_edge_list.append(e)
+        else:
+            xyzs = np.array([v.point.xyz for v in vs])
+            #repeat each point and remove the first and last point after the repeat
+            xyzs = np.repeat(xyzs, 2, axis=0)[1:-1]
+            xyzs = np.reshape(xyzs, (int(len(xyzs)/2), 2))
+            for e_xyz in xyzs:
+                v2 = create.vertex_list(e_xyz)
+                line_edge = create.pline_edge_frm_verts(v2)
+                line_edge_list.append(line_edge)
+
+    return line_edge_list 
+
+def edges2lines(edge_list: list[topobj.Edge]) -> np.ndarray:
     """
     converts edges to lines for visualisation.
  
     Parameters
     ----------
-    edge_list : list of edge object
+    edge_list : list[topobj.Edge]
         the list of edge object to convert to lines.
  
     Returns
     -------
-    vertices : shape(Npoints,3) of the vertiices
-        vertices of the lines
+    vertices : np.ndarray
+        np.ndarray(shape(Npoints,3)) vertices of the linea
     """
     all_xyzs = []
     
@@ -293,34 +338,45 @@ def edges2lines(edge_list):
     
     return all_xyzs 
 
-def move_topo(topo, target_xyz, ref_xyz = None):
+def move_topo(topo: topobj.Topology, target_xyz: list[float], ref_xyz: list[float] = None) -> topobj.Topology:
     """
     move topology to the target xyz
  
     Parameters
     ----------
-    topo : topo object
+    topo : topobj.Topology
         the topo object to move.
         
-    target_xyz: np.ndarray
+    target_xyz: list[float]
         target xyz position.
     
-    ref_xyz: np.ndarray
+    ref_xyz: list[float]
         reference xyz position, if not specified will use midpt of the topology.
  
     Returns
     -------
-    mv_topo : topo
+    mv_topo : topobj.Topology
         moved topo
     """
-    #TODO: account for different kind of geometries
+    #TODO: implement for bspline geometries
+    topotype = topo.topo_type
+    if topotype == topobj.TopoType.EDGE:
+        print('I am in bspline edge')
+        if topo.curve_type == geom.CurveType.BSPLINE:
+            print('not yet implemented for bspline curve')
+            return None
+    
+    if topotype == topobj.TopoType.FACE:
+        if topo.surface_type == geom.SrfType.BSPLINE:
+            print('not yet implemented for bspline surface')
+            return None
+
     mv_topo = copy.deepcopy(topo)
     vs = get.topo_explorer(mv_topo, topobj.TopoType.VERTEX)
-    if ref_xyz == None:
+    if ref_xyz is None:
         #find the midpt of the topo and use that as the ref xyz
         bbox = calculate.bbox_frm_topo(topo)
         ref_xyz = calculate.bbox_centre(bbox)
-        
     tx = target_xyz[0] - ref_xyz[0]
     ty = target_xyz[1] - ref_xyz[1]
     tz = target_xyz[2] - ref_xyz[2]
@@ -332,18 +388,90 @@ def move_topo(topo, target_xyz, ref_xyz = None):
     for cnt,v in enumerate(vs): v.point.xyz = trsf_xyzs[cnt]
     return mv_topo
 
-def reverse_face_normal(face):
+def rotate_topo(topo: topobj.Topology, axis: list[float], rotation: float, ref_xyz: list[float] = None):
+    """
+    rotate topology
+ 
+    Parameters
+    ----------
+    topo : topobj.Topology
+        the topo object to move.
+        
+    axis: list[float]
+        tuple specifying the axis to rotate along.
+    
+    rotation: float
+        rotation in degrees, anticlockwise.
+        
+    ref_xyz: list[float]
+        reference xyz position, if not specified will use midpt of the topology.
+ 
+    Returns
+    -------
+    mv_topo : topobj.Topology
+        moved topo
+    """
+    #TODO: implement for bspline geometries
+    topotype = topo.topo_type
+    if topotype == topobj.TopoType.EDGE:
+        if topo.curve_type == geom.CurveType.BSPLINE:
+            print('not yet implemented for bspline curve')
+            return None
+    
+    if topotype == topobj.TopoType.FACE:
+        if topo.surface_type == geom.SrfType.BSPLINE:
+            print('not yet implemented for bspline surface')
+            return None
+    
+    rot_topo = copy.deepcopy(topo)
+    if ref_xyz is None:
+        #find the midpt of the topo and use that as the ref xyz
+        bbox = calculate.bbox_frm_topo(topo)
+        ref_xyz = calculate.bbox_centre(bbox)
+    #move the topo to origin for the rotation
+    tx = 0 - ref_xyz[0]
+    ty = 0 - ref_xyz[1]
+    tz = 0 - ref_xyz[2]
+    trsl_mat = calculate.translate_matrice(tx, ty, tz)
+    rot_mat = calculate.rotate_matrice(axis, rotation)
+    trsl_mat2 = calculate.inverse_matrice(trsl_mat)
+    
+    vs = get.topo_explorer(rot_topo, topobj.TopoType.VERTEX)
+    xyzs = np.array([v.point.xyz for v in vs])
+    trsf_xyzs = calculate.trsf_xyzs(xyzs, trsl_mat2@rot_mat@trsl_mat)
+    # trsf_xyzs = calculate.trsf_xyzs(xyzs, trsl_mat)
+
+    for cnt,v in enumerate(vs): 
+        v.point.xyz = trsf_xyzs[cnt]
+    
+    #get all the surfaces and update their normals
+    if rot_topo.topo_type == topobj.TopoType.FACE:
+        faces = [rot_topo]
+    elif rot_topo.topo_type == topobj.TopoType.SHELL:
+        faces = get.faces_frm_solid(rot_topo)
+    elif rot_topo.topo_type == topobj.TopoType.SOLID:
+        faces = get.faces_frm_solid(rot_topo)
+    elif rot_topo.topo_type == topobj.TopoType.COMPOSITE:
+        faces = get.faces_frm_composite(rot_topo)
+    
+    if len(faces) != 0:
+        for f in faces:
+            f.update_polygon_surface()
+            
+    return rot_topo
+    
+def reverse_face_normal(face: topobj.Face) -> topobj.Face:
     """
     reverse the normal of the face
  
     Parameters
     ----------
-    face : topo object
+    face : topobj.Face
         the face to reverse.
         
     Returns
     -------
-    reversed_face : topo
+    reversed_face : topobj.Face
         the reversed face
     """
     #check the surface type of the face
@@ -354,7 +482,7 @@ def reverse_face_normal(face):
         bxyzs = np.flip(bxyzs, axis=0)
         bverts = create.vertex_list(bxyzs)
         if len(hverts) == 0:    
-            flip_f = create.polygon_face_frm_verts(bverts)
+            flip_f = create.polygon_face_frm_verts(bverts, attributes=face.attributes)
         else:
             hverts_ls = []
             for h in hverts:     
@@ -363,7 +491,8 @@ def reverse_face_normal(face):
                 hverts = create.vertex_list(hxyzs)    
                 hverts_ls.append(hverts)
             flip_f = create.polygon_face_frm_verts(bverts, 
-                                                   hole_vertex_list = hverts_ls)
+                                                   hole_vertex_list = hverts_ls,
+                                                   attributes = face.attributes)
         return flip_f
     
     elif face.surface_type == geom.SrfType.BSPLINE:
@@ -379,44 +508,44 @@ def reverse_face_normal(face):
         
         deg_u = srf.degree_u
         deg_v = srf.degree_v
-        flip_f = create.bspline_face_frm_ctrlpts(cp, kv_u, kv_v, deg_u, deg_v)
+        flip_f = create.bspline_face_frm_ctrlpts(cp, kv_u, kv_v, deg_u, deg_v, attributes=face.attributes)
         return flip_f 
 
-def update_topo_att(topo, attributes):
+def update_topo_att(topo: topobj.Topology, attributes: dict):
     """
     update the attributes of the topology
  
     Parameters
     ----------
-    topo : topo object
+    topo : topobj.Topology
         topology to update.
     
-    attributes: dictionary
+    attributes: dict
         the attributes
     """
     topo.update_attributes(attributes)
 
-def overwrite_topo_att(topo, attributes):
+def overwrite_topo_att(topo: topobj.Topology, attributes: dict):
     """
-    update the attributes of the topology
+    overwrite the attributes of the topology
  
     Parameters
     ----------
-    topo : topo object
+    topo : topobj.Topology
         topology to update.
     
-    attributes: dictionary
+    attributes: dict
         the attributes
     """
     topo.overwrite_attributes(attributes)
 
-def xyzs2voxs(xyzs, xdim, ydim, zdim):
+def xyzs2voxs(xyzs: np.ndarray, xdim: float, ydim: float, zdim: float) -> dict:
     """
     This function group the vertices into a 3d voxel grid.
     
     Parameters
     ----------
-    xyzs1 : ndarray
+    xyzs : np.ndarray
         array defining the point.
     
     xdim : float
@@ -430,7 +559,7 @@ def xyzs2voxs(xyzs, xdim, ydim, zdim):
     
     Returns
     -------
-    voxel_dictionary : a dictionary of voxels
+    voxel_dictionary : dict
         a dictionary with key (i,j,k) for each voxel and the value as the index of the point
     """
     if type(xyzs) != np.ndarray:
@@ -449,6 +578,7 @@ def xyzs2voxs(xyzs, xdim, ydim, zdim):
     j = np.fix((y - mny)/ydim)
     k = np.fix((z - mnz)/zdim)
     ijks = np.stack((i,j,k), axis=-1)
+    vox_props = {'voxel_dim': [xdim, ydim, zdim]}
     voxs = {}
     for cnt,ijk in enumerate(ijks):
         ijk = ijk.tolist()
@@ -461,8 +591,8 @@ def xyzs2voxs(xyzs, xdim, ydim, zdim):
             mpy = ijk[1] * ydim + (ydim/2) + mny
             mpz = ijk[2] * zdim + (zdim/2) + mnz
             voxs[ijk] = {'idx':[cnt], 'midpt':[mpx,mpy,mpz]}
-            
-    return voxs
+    vox_props['voxels'] = voxs
+    return vox_props
 
-def trsf_cs(cs1, cs2, topo):
-    pass
\ No newline at end of file
+def trsf_cs(cs1: utility.CoordinateSystem, cs2: utility.CoordinateSystem, topo: topobj.Topology) -> topobj.Topology:
+    pass
